#ifndef id873715F57B504CCF8227CE03EA28CAFA
#define id873715F57B504CCF8227CE03EA28CAFA

#include "doorkeeper/primitivetypes.hpp"
#include "doorkeeper/implem/helpers.hpp"
#include <boost/iterator/iterator_facade.hpp>
#include <iterator>
#include <vector>
#include <type_traits>
#include <cstdint>
#include <cassert>

namespace dk {
	template <typename T, uint32_t D, typename T1>
	class TileIterator;

	namespace implem {
		template <uint32_t D>
		size_t get_index_from_pos ( const Vector<D>& parPos, const Vector<D>& parSize ) a_pure;

		template <typename I, typename O>
		struct TypeWithQualifiers {
			typedef typename std::conditional<std::is_volatile<I>::value && std::is_const<I>::value,
						typename std::add_cv<O>::type,
						typename std::conditional<std::is_volatile<I>::value,
							typename std::add_volatile<typename std::remove_cv<O>::type>::type,
							typename std::conditional<std::is_const<I>::value,
								typename std::add_const<typename std::remove_cv<O>::type>::type,
								typename std::remove_cv<O>::type
							>::type
						>::type
					>::type result;
		};

#if defined(NDEBUG)
		template <>
		size_t get_index_from_pos<2> ( const Vector<2>& parPos, const Vector<2>& parSize ) a_pure;
#endif

		template <uint32_t D>
		Vector<D> buildPastEndCoordinate ( const Vector<D>& parFrom, const Vector<D>& parTo ) a_pure;

		template <typename T, uint32_t D, typename T1>
		const Vector<D>& get_from_from_iterator ( const TileIterator<T, D, T1>& parIterator );
	} //namespace implem

	template <typename T, uint32_t D, typename T1=typename std::remove_cv<T>::type>
	class TileIterator : public boost::iterator_facade<TileIterator<T, D>, T, boost::bidirectional_traversal_tag> {
		friend class boost::iterator_core_access;
		friend const Vector<D>& implem::get_from_from_iterator<> ( const TileIterator& parIterator );
		typedef std::vector<T1> vector_type;
		typedef typename implem::TypeWithQualifiers<T, vector_type>::result qualif_vector_type;
	public:
		typedef Vector<D> coords;

		TileIterator ( void );
		TileIterator ( const TileIterator& parOther ) = default;
		TileIterator ( TileIterator&& parOther ) = default;
		TileIterator ( qualif_vector_type* parData, const coords& parFrom, const coords& parTo );
		TileIterator ( qualif_vector_type* parData, const coords& parFrom, const coords& parTo, const coords& parAreaFrom, const coords& parAreaTo );
		TileIterator ( qualif_vector_type* parData, const coords& parFrom, const coords& parTo, const coords& parAreaFrom, const coords& parAreaTo, const coords& parSubdiv );
		~TileIterator ( void ) = default;

		const coords& abs_position ( void ) const { return m_pos; }

	private:
		void increment ( void );
		void decrement ( void );
		void advance ( size_t parAdvance );
		ptrdiff_t distance_to ( const TileIterator& parOther );
		bool equal ( const TileIterator& parOther ) const;
		T& dereference ( void ) const;
		size_t get_current_index ( void ) const { return implem::get_index_from_pos<D>(m_pos, m_subdivareasize) + m_subindex; }

		coords m_pos;
		coords m_from;
		coords m_to;
		coords m_subdivareasize;
		qualif_vector_type* m_data;
		CoordinateScalarType m_subindex;
		CoordinateScalarType m_maxsubindex;
	};
} //namespace dk

#include "doorkeeper/implem/tileiterator.inl"

#endif
